@@ -18,10 +18,6 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/*
- * $Id$
- */
-
 #ifdef __APPLE__
 #include <pthread.h>
 #endif
@@ -239,13 +235,18 @@ typedef struct
 #define DELLSCRK    0x413C2101
 #define DELLSK      0x413C2100
 #define KOBIL_TRIBANK	0x0D463010
-#define KOBIL_MIDENTITY_VISUAL	0x0D460D46
+#define KOBIL_MIDENTITY_VISUAL	0x0D464289
 #define VEGAALPHA   0x09820008
 #define HPSMARTCARDKEYBOARD 0x03F01024
 #define HP_CCIDSMARTCARDKEYBOARD 0x03F00036
 #define KOBIL_IDTOKEN 0x0D46301D
 #define FUJITSUSMARTKEYB 0x0BF81017
 #define FEITIANR502DUAL 0x096E060D
+#define MICROCHIP_SEC1100 0x04241104
+#define CHERRY_KC1000SC 0x046A00A1
+#define ElatecTWN4	0x09D80427
+#define SCM_SCL011 0x04E65293
+#define HID_AVIATOR	0x076B3A21
 
 // CCID readers
 #define ACS_ACR32_ICC_READER			0x072fb301
