@@ -1,7 +1,7 @@
 /*
     ccid_ifdhandler.h: non-generic ifdhandler functions
     Copyright (C) 2004-2010   Ludovic Rousseau
-    Copyright (C) 2010-2014   Advanced Card Systems Ltd.
+    Copyright (C) 2010-2016   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,10 +18,6 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/*
- * $Id$
- */
-
 #ifndef _ccid_ifd_handler_h_
 #define _ccid_ifd_handler_h_
 
@@ -54,10 +50,19 @@
 // ACR128 I/O control code for escape command
 #define IOCTL_ACR128_READER_COMMAND	SCARD_CTL_CODE(2079)
 
+/* Control codes for Windows compatibility. */
+#define WINSCARD_CTL_CODE(code)					(0x310000 | (code << 2))
+#define WINIOCTL_CCID_ESCAPE					WINSCARD_CTL_CODE(3500)
+#define WINIOCTL_SMARTCARD_GET_FIRMWARE_VERSION	WINSCARD_CTL_CODE(2078)
+#define WINIOCTL_SMARTCARD_DISPLAY_LCD_MESSAGE	WINSCARD_CTL_CODE(2079)
+#define WINIOCTL_SMARTCARD_READ_KEY				WINSCARD_CTL_CODE(2080)
+#define WINIOCTL_SMARTCARD_SET_CARD_TYPE		WINSCARD_CTL_CODE(2060)
+#define WINIOCTL_SMARTCARD_SET_CARD_VOLTAGE		WINSCARD_CTL_CODE(2074)
+#define WINIOCTL_ACR128_READER_COMMAND			WINSCARD_CTL_CODE(2079)
+
 #define DRIVER_OPTION_CCID_EXCHANGE_AUTHORIZED 1
 #define DRIVER_OPTION_GEMPC_TWIN_KEY_APDU 2
 #define DRIVER_OPTION_USE_BOGUS_FIRMWARE 4
-#define DRIVER_OPTION_RESET_ON_CLOSE 8
 #define DRIVER_OPTION_DISABLE_PIN_RETRIES (1 << 6)
 
 // ACS driver option
