@@ -1,7 +1,7 @@
 /*
     ifdhandler.c: IFDH API
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2015   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2016   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,8 +18,6 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/* $Id$ */
-
 #include <config.h>
 
 #ifdef HAVE_STDIO_H
@@ -90,6 +88,22 @@ static RESPONSECODE process_spe_ppdu(unsigned int reader_index,
 	unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength);
 
+static void FreeChannel(int reader_index)
+{
+#ifdef HAVE_PTHREAD
+	(void)pthread_mutex_lock(&ifdh_context_mutex);
+#endif
+
+	(void)ClosePort(reader_index);
+	ReleaseReaderIndex(reader_index);
+
+	free(CcidSlots[reader_index].readerName);
+	memset(&CcidSlots[reader_index], 0, sizeof(CcidSlots[reader_index]));
+
+#ifdef HAVE_PTHREAD
+	(void)pthread_mutex_unlock(&ifdh_context_mutex);
+#endif
+}
 
 static RESPONSECODE CreateChannelByNameOrChannel(DWORD Lun,
 	LPSTR lpcDevice, DWORD Channel)
@@ -209,7 +223,13 @@ static RESPONSECODE CreateChannelByNameOrChannel(DWORD Lun,
 		oldReadTimeout = ccid_descriptor->readTimeout;
 
 		/* 100 ms just to resync the USB toggle bits */
-		ccid_descriptor->readTimeout = 100;
+		/* Do not use a fixed 100 ms value but compute it from the
+		 * default timeout. It is now possible to use a different value
+		 * by changing readTimeout in ccid_open_hack_pre()
+		 * ccid_descriptor->readTimeout = ccid_descriptor->readTimeout * 100.0 / DEFAULT_COM_READ_TIMEOUT; */
+
+		/* Avoid libusb timeout on Mac OS X. */
+		ccid_descriptor->readTimeout = 1000;
 
 		if ((IFD_COMMUNICATION_ERROR == CcidSlots[reader_index].pGetSlotStatus(reader_index, pcbuffer))
 			&& (IFD_COMMUNICATION_ERROR == CcidSlots[reader_index].pGetSlotStatus(reader_index, pcbuffer)))
@@ -239,8 +259,7 @@ error:
 	if (return_value != IFD_SUCCESS)
 	{
 		/* release the allocated resources */
-		free(CcidSlots[reader_index].readerName);
-		ReleaseReaderIndex(reader_index);
+		FreeChannel(reader_index);
 	}
 	else
 	{
@@ -333,19 +352,7 @@ EXTERNAL RESPONSECODE IFDHCloseChannel(DWORD Lun)
 	(void)CcidSlots[reader_index].pPowerOff(reader_index);
 	/* No reader status check, if it failed, what can you do ? :) */
 
-#ifdef HAVE_PTHREAD
-	(void)pthread_mutex_lock(&ifdh_context_mutex);
-#endif
-
-	(void)ClosePort(reader_index);
-	ReleaseReaderIndex(reader_index);
-
-	free(CcidSlots[reader_index].readerName);
-	memset(&CcidSlots[reader_index], 0, sizeof(CcidSlots[reader_index]));
-
-#ifdef HAVE_PTHREAD
-	(void)pthread_mutex_unlock(&ifdh_context_mutex);
-#endif
+	FreeChannel(reader_index);
 
 	return IFD_SUCCESS;
 } /* IFDHCloseChannel */
@@ -664,7 +671,7 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 				if (ccid_desc->sIFD_serial_number)
 				{
 					strlcpy((char *)Value, ccid_desc->sIFD_serial_number, *Length);
-					*Length = strlen((char *)Value);
+					*Length = strlen((char *)Value)+1;
 				}
 				else
 				{
@@ -1538,11 +1545,13 @@ EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 	if (-1 == (reader_index = LunToReaderIndex(Lun)))
 		return IFD_COMMUNICATION_ERROR;
 
+	ccid_descriptor = get_ccid_descriptor(reader_index);
+
 	DEBUG_INFO3("%s (lun: " DWORD_X ")", CcidSlots[reader_index].readerName,
 		Lun);
 
 	/* special APDU for the Kobil IDToken (CLASS = 0xFF) */
-	if (KOBIL_IDTOKEN == get_ccid_descriptor(reader_index) -> readerID)
+	if (KOBIL_IDTOKEN == ccid_descriptor -> readerID)
 	{
 		char manufacturer[] = {0xFF, 0x9A, 0x01, 0x01, 0x00};
 		char product_name[] = {0xFF, 0x9A, 0x01, 0x03, 0x00};
@@ -1570,7 +1579,7 @@ EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 		if ((sizeof firmware_version == TxLength)
 			&& (memcmp(TxBuffer, firmware_version, sizeof firmware_version) == 0))
 		{
-			int IFD_bcdDevice = get_ccid_descriptor(reader_index)->IFD_bcdDevice;
+			int IFD_bcdDevice = ccid_descriptor -> IFD_bcdDevice;
 
 			DEBUG_INFO1("IDToken: Firmware version command");
 			*RxLength = sprintf((char *)RxBuffer, "%X.%02X",
@@ -1592,8 +1601,6 @@ EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 
 	}
 
-	ccid_descriptor = get_ccid_descriptor(reader_index);
-
 	// Fix reader hang problem by checking card status of ACR85 PICC before exchanging APDU
 	if ((ccid_descriptor->readerID == ACS_ACR85_PINPAD_READER_PICC) &&
 		(ccid_descriptor->firmwareFixEnabled))
@@ -1763,7 +1770,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		(ACS_AET65_1SAM_ICC_READER == ccid_descriptor -> readerID))
 	{
 		// Set card voltage
-		if (IOCTL_SMARTCARD_SET_CARD_VOLTAGE == dwControlCode)
+		if ((IOCTL_SMARTCARD_SET_CARD_VOLTAGE == dwControlCode)
+			|| (WINIOCTL_SMARTCARD_SET_CARD_VOLTAGE == dwControlCode))
 		{
 			unsigned int iBytesReturned;
 
@@ -1774,7 +1782,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		}
 
 		// Set card type
-		if (IOCTL_SMARTCARD_SET_CARD_TYPE == dwControlCode)
+		if ((IOCTL_SMARTCARD_SET_CARD_TYPE == dwControlCode)
+			|| (WINIOCTL_SMARTCARD_SET_CARD_TYPE == dwControlCode))
 		{
 			unsigned int iBytesReturned;
 
@@ -2194,7 +2203,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	}
 
 	// MS CCID I/O control code for escape command
-	if (IOCTL_CCID_ESCAPE == dwControlCode)
+	if ((IOCTL_CCID_ESCAPE == dwControlCode)
+		|| (WINIOCTL_CCID_ESCAPE == dwControlCode))
 	{
 		unsigned int iBytesReturned;
 
@@ -2212,7 +2222,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		(ACS_APG8201Z2 == ccid_descriptor -> readerID))
 	{
 		// Get firmware version
-		if (IOCTL_SMARTCARD_GET_FIRMWARE_VERSION == dwControlCode)
+		if ((IOCTL_SMARTCARD_GET_FIRMWARE_VERSION == dwControlCode)
+			|| (WINIOCTL_SMARTCARD_GET_FIRMWARE_VERSION == dwControlCode))
 		{
 			unsigned char command[] = { 0x04, 0x00, 0x00, 0x00, 0x00 };
 			unsigned int commandLen = sizeof(command);
@@ -2237,7 +2248,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		}
 
 		// Display LCD message
-		if (IOCTL_SMARTCARD_DISPLAY_LCD_MESSAGE == dwControlCode)
+		if ((IOCTL_SMARTCARD_DISPLAY_LCD_MESSAGE == dwControlCode)
+			|| (WINIOCTL_SMARTCARD_DISPLAY_LCD_MESSAGE == dwControlCode))
 		{
 			unsigned char command[5 + 32] = { 0x05, 0x00, 0x20, 0x00, 0x00 };
 			unsigned int commandLen = sizeof(command);
@@ -2271,7 +2283,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		}
 
 		// Read key
-		if (IOCTL_SMARTCARD_READ_KEY == dwControlCode)
+		if ((IOCTL_SMARTCARD_READ_KEY == dwControlCode)
+			|| (WINIOCTL_SMARTCARD_READ_KEY == dwControlCode))
 		{
 			unsigned char command[5 + 6] = { 0x06, 0x00, 0x06, 0x00, 0x00 };
 			unsigned int commandLen = sizeof(command);
@@ -2304,7 +2317,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	else
 	{
 		// ACR128 I/O control code for escape command
-		if (IOCTL_ACR128_READER_COMMAND == dwControlCode)
+		if ((IOCTL_ACR128_READER_COMMAND == dwControlCode)
+			|| (WINIOCTL_ACR128_READER_COMMAND == dwControlCode))
 		{
 			unsigned char *command;
 			unsigned int commandLen = 3 + TxLength;
